<?php
namespace Datacake\Core\Includes;
/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       http://www.datacake.com.br
 * @since      1.0.0
 *
 * @package    Datacake_core
 * @subpackage Datacake_core/includes
 */

use Datacake\Core\Includes\DatacakeCoreLoader;
use Datacake\Core\Includes\DatacakeCoreI18n;
use Datacake\Core\Admin\DatacakeCoreAdmin;
use Datacake\Core\Admin\Meta\IspMetaBox;
use Datacake\Core\Pub\DatacakeCorePublic;


// Load Api Views
use Datacake\Core\Admin\ApiViews\CityApiView;
use Datacake\Core\Admin\ApiViews\ProductCategoryApiView;
use Datacake\Core\Admin\ApiViews\ProductTagApiView;
use Datacake\Core\Admin\ApiViews\ProductApiView;
use Datacake\Core\Admin\ApiViews\ChannelGroupApiView;
use Datacake\Core\Admin\ApiViews\ChannelTypeApiView;
use Datacake\Core\Admin\ApiViews\ChannelApiView;


/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Datacake_core
 * @subpackage Datacake_core/includes
 * @author     Datacake <ajuda@datacake.com.br>
 */
class DatacakeCore {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      DatacakeCoreLoader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'DATACAKE_CORE_VERSION' ) ) {
			$this->version = DATACAKE_CORE_VERSION;
		} else {
			$this->version = '1.1.0';
		}
		$this->plugin_name = 'datacake_core';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();
		
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - DatacakeCoreLoader. Orchestrates the hooks of the plugin.
	 * - DatacakeCoreI18n. Defines internationalization functionality.
	 * - DatacakeCoreAdmin. Defines all hooks for the admin area.
	 * - DatacakeCorePublic. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		

		$this->loader = new DatacakeCoreLoader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the DatacakeCoreI18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new DatacakeCoreI18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new DatacakeCoreAdmin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'admin_menu' );
		
		$this->loader->add_filter( 'w3tc_pgcache_cookiegroups', $plugin_admin, 'w3tc_pgcache_cookiegroups_filter');
		// Add Plan MetaBoxes
		$plan_metabox = new IspMetaBox( $this->get_plugin_name(), $this->get_version() );
		// Adding Custom Meta Blocks
		$this->loader->add_action( 'add_meta_boxes', $plan_metabox, 'add_meta_boxes' );
		
		// Loading Api Views For Admins
		
		$CityApiView = new CityApiView();
		$CityApiView->register_actions($this->loader);
		// Products
		$ProductCategoryView = new ProductCategoryApiView();
		$ProductCategoryView->register_actions($this->loader);
		$ProductTagView = new ProductTagApiView();
		$ProductTagView->register_actions($this->loader);
		$ProductView = new ProductApiView();
		$ProductView->register_actions($this->loader);
		// Channels
		$ChannelGroupApiView = new ChannelGroupApiView();
		$ChannelGroupApiView->register_actions($this->loader);
		$ChannelTypeApiView = new ChannelTypeApiView();
		$ChannelTypeApiView->register_actions($this->loader);
		$ChannelApiView = new ChannelApiView();
		$ChannelApiView->register_actions($this->loader);
		
	}
	
	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new DatacakeCorePublic( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
		// Getting Cookies
		$this->loader->add_action('init', $plugin_public, 'register_post_types');
		$this->loader->add_action('init', $plugin_public, 'set_cookie_based_on_get');
		// Making Redirect
		$this->loader->add_action('pre_get_posts', $plugin_public, 'page_redirects');
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    DatacakeCoreLoader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
